# ALMA Data Reduction Script

# Adapted for ERIS2017 Polarization Tutorial (Marti-Vidal, Oct. 2017)





# Calibration



thesteps = []

step_title = {1: 'Listobs, and save original flags',

              2: 'Initial flagging',

              3: 'Putting a model for the flux calibrator(s)',

              4: 'Save flags before bandpass cal',

              5: 'Bandpass calibration',

              6: 'Save flags before gain cal',

              7: 'Gain calibration',

              8: 'Save flags before applycal',

              9: 'Application of the bandpass and gain cal tables',

              10: 'Split out corrected column',

              11: 'Save flags after applycal'}







if 'applyonly' not in globals(): applyonly = False

try:

  print 'List of steps to be executed ...', mysteps

  thesteps = mysteps

except:

  print 'global variable mysteps not set.'

if (thesteps==[]):

  thesteps = range(0,len(step_title))

  print 'Executing all steps: ', thesteps



# The Python variable 'mysteps' will control which steps

# are executed when you start the script using

#   execfile('scriptForCalibration.py')

# e.g. setting

#   mysteps = [2,3,4]# before starting the script will make the script execute

# only steps 2, 3, and 4

# Setting mysteps = [] will make it execute all steps.















import re

import os



#import ERIS2017_helper as ERIS2017

execfile('ERIS2017_helper.py')









# CALIBRATE_BANDPASS: J2258-2758, J0635-7516

# CALIBRATE_FLUX: J0522-3627

# CALIBRATE_PHASE: J0747-3310

# OBSERVE_CHECK: J0725-2640

# OBSERVE_TARGET: VY_CMa







VIS = 'ERIS_2017.Polarization_Tutorial.DATA.ms'

REFANT = 'DV12'



POLCAL = 'J0522-3627'



TARGET = 'VY_CMa, J0725-2640' # TARGET PLUS CHECK SOURCE!

PHASECAL = 'J0747-3310'

BANDPASSCAL = 'J2258-2758, J0635-7516'  # TWO BP CALIBRATORS!





FLUXCAL = 'J0522-3627'

FLUX = 3.268

SPIX = -0.351

FREQ = '177GHz'









T = True

F = False































# Listobs, and save original flags

mystep = 1

if(mystep in thesteps):

  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

  print 'Step ', mystep, step_title[mystep]



  os.system('rm -rf %s.listobs'%VIS)

  listobs(vis = VIS,

    listfile = '%s.listobs'%VIS)

  

  if not os.path.exists('%s.flagversions/flags.Original'%VIS):  

    flagmanager(vis = VIS,

      mode = 'save',

      versionname = 'Original')

  

  

#  ERIS2017.plotSchedule(VIS, bandpass=BANDPASSCAL, fluxcal=FLUXCAL,

#               phasecal=PHASECAL, target=TARGET, polcal=POLCAL)

  plotSchedule(VIS, bandpass=BANDPASSCAL, fluxcal=FLUXCAL,

               phasecal=PHASECAL, target=TARGET, polcal=POLCAL)





  plotants(VIS,figfile='%s.PLOTANTS.png'%VIS)





















# Initial flagging

mystep = 2

if(mystep in thesteps):

  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

  print 'Step ', mystep, step_title[mystep]



  # Flagging shadowed data

  flagdata(vis = VIS,

    mode = 'shadow',

    flagbackup = F)

  

  # Bad antennas

  flagdata(vis = VIS,

    mode = 'manual',

    antenna = 'DV05,DA58',

    flagbackup = F)





































# Putting a model for the flux calibrator(s)

mystep = 3

if(mystep in thesteps):

  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

  print 'Step ', mystep, step_title[mystep]



  setjy(vis = VIS,

    standard = 'manual',

    field = FLUXCAL,

    fluxdensity = [FLUX, 0, 0, 0],

    spix = SPIX,

    reffreq = FREQ)



  

































# Save flags before bandpass cal

mystep = 4

if(mystep in thesteps):

  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

  print 'Step ', mystep, step_title[mystep]



  Vname = 'BeforeBandpassCalibration'



  if os.path.exists('%s.flagversions/flags.%s'%(VIS,Vname)):

    print 'WILL DELETE PREVIOUS FLAG VERSION:\n%s'%Vname

    flagmanager(vis = VIS,

      mode = 'delete',

      versionname = Vname)



  flagmanager(vis = VIS,

    mode = 'save',

    versionname = Vname)

  

  

































# Bandpass calibration

mystep = 5

if(mystep in thesteps):

  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

  print 'Step ', mystep, step_title[mystep]



  os.system('rm -rf %s.ap_pre_bandpass'%VIS) 

  

  gaincal(vis = VIS,

    caltable = '%s.ap_pre_bandpass'%VIS,

    field = BANDPASSCAL, 

    spw = '',

    scan = '',

    solint = 'int', # WHY!!!!

    refant = REFANT,

    calmode = 'p')

 



#  ERIS2017.plot_G('%s.ap_pre_bandpass'%VIS,toplot=['phase'])

  plot_G('%s.ap_pre_bandpass'%VIS,toplot=['phase'])

  



  bandpass(vis = VIS,

    caltable = '%s.bandpass'%VIS,

    field = BANDPASSCAL, 

    scan = '',

    solint = 'inf', # WHY!!!!

#######

# CHOOSE:

    combine = 'scan,field,obs',

#    combine = 'scan,field',

#    combine = 'scan',

#######

    refant = REFANT,

    solnorm = True, # WHY!!!!

    bandtype = 'B',

    gaintable = ['%s.ap_pre_bandpass'%VIS])





  

  os.system('rm -rf %s.bandpass_smooth20ch'%VIS) 

 



  bandpass(vis = VIS,

    caltable = '%s.bandpass_smooth20ch'%VIS,

    field = BANDPASSCAL, 

    scan = '',

    spw = '0',

    solint = 'inf,20ch', # WHY!!!!

#######

# CHOOSE:

    combine = 'scan,field,obs',

#    combine = 'scan,field',

#    combine = 'scan',

#######

    refant = REFANT,

    solnorm = True, # WHY!!!!

    bandtype = 'B',

    gaintable = ['%s.ap_pre_bandpass'%VIS])

 





#  ERIS2017.plot_B('%s.bandpass'%VIS, '%s.bandpass_smooth20ch'%VIS)

  plot_B('%s.bandpass'%VIS, '%s.bandpass_smooth20ch'%VIS)



  







































# Save flags before gain cal

mystep = 6

if(mystep in thesteps):

  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

  print 'Step ', mystep, step_title[mystep]



  Vname = 'BeforeGainCalibration'



  if os.path.exists('%s.flagversions/flags.%s'%(VIS,Vname)):

    print 'WILL DELETE PREVIOUS FLAG VERSION:\n%s'%Vname

    flagmanager(vis = VIS,

      mode = 'delete',

      versionname = Vname)



  flagmanager(vis = VIS,

    mode = 'save',

    versionname = Vname)

  

  

  











































# Gain calibration

mystep = 7

if(mystep in thesteps):

  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

  print 'Step ', mystep, step_title[mystep]



  os.system('rm -rf %s.phase_int'%VIS) 







  gaincal(vis = VIS,

    caltable = '%s.phase_int'%VIS,

    field = ','.join([BANDPASSCAL,PHASECAL,FLUXCAL]), # WHY!!!!

    solint = 'int',

    refant = REFANT,

##########

# CHOOSE 

    gaintype = 'G',

#    gaintype = 'T',

########## 

    calmode = 'p', # !!!!

##########

# CHOOSE ONE OF THESE TWO:

#    gaintable = ['%s.bandpass_smooth20ch'%VIS])

    gaintable = ['%s.bandpass'%VIS])

########## 





#  ERIS2017.plot_G('%s.phase_int'%VIS, toplot=['phase'])

  plot_G('%s.phase_int'%VIS, toplot=['phase'])



  

  os.system('rm -rf %s.ampli_inf'%VIS) 

  gaincal(vis = VIS,

    caltable = '%s.ampli_inf'%VIS,

    field = ','.join([BANDPASSCAL,PHASECAL,FLUXCAL]),

    solint = 'inf', # WHY!!!!

    refant = REFANT,

#########

# CHOOSE:

    gaintype = 'T',

#    gaintype = 'G',

#########

    calmode = 'a',

#########

# CHOOSE:

#    gaintable = ['%s.bandpass_smooth20ch'%VIS, '%s.phase_int'%VIS])

    gaintable = ['%s.bandpass'%VIS, '%s.phase_int'%VIS])

#########

 

  

  os.system('rm -rf %s.flux_inf'%VIS) 

  os.system('rm -rf %s.fluxscale'%VIS) 



  mylogfile = casalog.logfile()

  casalog.setlogfile('%s.fluxscale'%VIS)



  fluxscaleDict = fluxscale(vis = VIS,

    caltable = '%s.ampli_inf'%VIS,

    fluxtable = '%s.flux_inf'%VIS,

    reference = FLUXCAL)

  

  casalog.setlogfile(mylogfile)



#  ERIS2017.plot_G('%s.ampli_inf'%VIS, '%s.flux_inf'%VIS, toplot=['amp']) 

  plot_G('%s.ampli_inf'%VIS, '%s.flux_inf'%VIS, toplot=['amp']) 





  

  os.system('rm -rf %s.phase_inf'%VIS) 

  gaincal(vis = VIS,

    caltable = '%s.phase_inf'%VIS,

    field = PHASECAL,

    solint = 'inf',  # WHY!!!!

    refant = REFANT,

########

# CHOOSE:

    gaintype = 'G',

#    gaintype = 'T',

########

    calmode = 'p',

########

# CHOOSE:

#    gaintable = ['%s.bandpass_smooth20ch'%VIS])

    gaintable = ['%s.bandpass'%VIS])

########





#  ERIS2017.plot_G('%s.phase_int'%VIS, '%s.phase_inf'%VIS, toplot=['phase','cross-phase'])

  plot_G('%s.phase_int'%VIS, '%s.phase_inf'%VIS, toplot=['phase','cross-phase'])







##################3

# WHAT IS THIS??!!



#  os.system('rm -rf %s.phase_int_zphs'%VIS)

#  os.system('cp -r  %s.phase_int %s.phase_int_zphs'%(VIS,VIS))

#  zeroRefant('%s.phase_int_zphs'%VIS,refant=REFANT)

#  plot_G('%s.phase_int_zphs'%VIS,'%s.phase_inf'%VIS, toplot=['phase'])

#  ERIS2017.zeroRefant('%s.phase_int_zphs'%VIS,refant=REFANT)

#  ERIS2017.plot_G('%s.phase_int_zphs'%VIS,'%s.phase_inf'%VIS, toplot=['phase'])



































  



# Save flags before applycal

mystep = 8

if(mystep in thesteps):

  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

  print 'Step ', mystep, step_title[mystep]



  Vname = 'BeforeApplycal'



  if os.path.exists('%s.flagversions/flags.%s'%(VIS,Vname)):

    print 'WILL DELETE PREVIOUS FLAG VERSION:\n%s'%Vname

    flagmanager(vis = VIS,

      mode = 'delete',

      versionname = Vname)



  flagmanager(vis = VIS,

    mode = 'save',

    versionname = Vname)

  

  

  







































# Application of the bandpass and gain cal tables

mystep = 9

if(mystep in thesteps):

  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

  print 'Step ', mystep, step_title[mystep]



  for fld in [BANDPASSCAL, FLUXCAL, PHASECAL]: 

    applycal(vis = VIS,

      field = fld,

########

# CHOOSE:

#      gaintable = ['%s.bandpass_smooth20ch'%VIS, '%s.split.phase_int'%VIS, '%s.split.flux_inf'%VIS],

#      gaintable = ['%s.bandpass'%VIS, '%s.phase_int_zphs'%VIS, '%s.flux_inf'%VIS],

      gaintable = ['%s.bandpass'%VIS, '%s.phase_int'%VIS, '%s.flux_inf'%VIS],

#      gaintable = ['%s.bandpass'%VIS, '%s.split.phase_inf'%VIS, '%s.split.flux_inf'%VIS],

#      gaintable = ['%s.bandpass_smooth20ch'%VIS, '%s.split.phase_inf'%VIS, '%s.split.flux_inf'%VIS],

########

      interp = 'linear,linear,linear,linear',

      calwt = T,

      flagbackup = F)

  





  applycal(vis = VIS,

    field = TARGET,

########

# CHOOSE:

#    gaintable = ['%s.bandpass_smooth20ch'%VIS, '%s.phase_inf'%VIS, '%s.flux_inf'%VIS],

    gaintable = ['%s.bandpass'%VIS, '%s.phase_inf'%VIS, '%s.flux_inf'%VIS],

#    gaintable = ['%s.bandpass_smooth20ch'%VIS, '%s.phase_int'%VIS, '%s.flux_inf'%VIS],

#    gaintable = ['%s.bandpass'%VIS, '%s.phase_int'%VIS, '%s.flux_inf'%VIS],

########

    gainfield = ['','', PHASECAL, PHASECAL], 

    interp = 'linear,linear,linear,linear',

    calwt = T,

    flagbackup = F)

  



# Split out corrected column

mystep = 10

if(mystep in thesteps):

  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

  print 'Step ', mystep, step_title[mystep]



  os.system('rm -rf %s.cal'%VIS) 

  os.system('rm -rf %s.cal.flagversions'%VIS) 

  

  listOfIntents = ['CALIBRATE_BANDPASS#ON_SOURCE',

   'CALIBRATE_FLUX#ON_SOURCE',

   'CALIBRATE_PHASE#ON_SOURCE',

   'CALIBRATE_POLARIZATION#ON_SOURCE',

   'CALIBRATE_WVR#ON_SOURCE',

   'OBSERVE_CHECK_SOURCE#ON_SOURCE',

   'OBSERVE_TARGET#ON_SOURCE']

  

  split(vis = VIS,

    outputvis = '%s.cal'%VIS,

    datacolumn = 'corrected',

    intent = ','.join(listOfIntents),

    keepflags = T)

  

  





































# Save flags after applycal

mystep = 11

if(mystep in thesteps):

  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

  print 'Step ', mystep, step_title[mystep]





  Vname = 'AfterApplycal'



  if os.path.exists('%s.flagversions/flags.%s'%(VIS,Vname)):

    print 'WILL DELETE PREVIOUS FLAG VERSION:\n%s'%Vname

    flagmanager(vis = VIS,

      mode = 'delete',

      versionname = Vname)

  

  flagmanager(vis = VIS,

    mode = 'save',

    versionname = Vname)

  

  



