# ALMA Data Reduction/Demonstration Script

# POLARIZATION CALIBRATION - VERSION DATE: 14-10-2017 - Rev. 8



# REVISED by gmoellen (2016Sep12) to calibrate the X/Y amp ratio

#  in the science target and phase calibrator



# REVISED by marti-vidal (Oct2016) for the B5 Science Verification



# ADAPTED by marti-vidal (Oct2017) for ERIS 2017 Pol. tutorial.









# Calibration





thesteps = []

step_title = {0: 'Listobs and save original flags. Clearcal.',

              1: 'Additional flagging',

              2: 'Polarization calibration I: X/Y Gain Ratio (G-mode)',

              3: 'Polarization calibration II: X/Y Cross-delay',

              4: 'Polarization calibration III: X-Y Cross-phase (Ambiguous)',

              5: 'Polarization calibration IV: X-Y Cross-phase (Unambiguous)',

              6: 'Polarization calibration V: D-terms',

              7: 'Polarization calibration VI: X/Y Amplitude Ratio',

              8: 'Save flags before applycal',

              9: 'Apply polarization calibration',

              10: 'Save flags after applycal',

              11: 'Clean the polarization calibrator',

              12: 'Show the full-polarization image'}





try:

  print 'List of steps to be executed ...', mysteps

  thesteps = mysteps

except:

  print 'global variable mysteps not set.'

if (thesteps==[]):

  thesteps = range(0,len(step_title))

  print 'Executing all steps: ', thesteps



# The Python variable 'mysteps' will control which steps

# are executed when you start the script using

#   execfile('scriptForCalibration.py')

# e.g. setting

#   mysteps = [2,3,4]# before starting the script will make the script execute

# only steps 2, 3, and 4

# Setting mysteps = [] will make it execute all steps.



import re

import pylab as pl

import numpy as np

import os

import pickle as pk



#import ERIS2017_helper as ERIS2017

execfile('ERIS2017_helper.py')



import recipes.almahelpers as ah

import recipes.almapolhelpers as aph



T = True

F = False





# Using reference antenna = DV12



VIS = 'ERIS_2017.Polarization_Tutorial.DATA.ms.cal'

REFANT = 'DV12'

POLCAL = 'J0522-3627'

TARGET = 'VY_CMa, J0725-2640' # TARGET PLUS CHECK SOURCE!

PHASECAL = 'J0747-3310'





#############

# FOR CLEAN OF THE POL. CALIBRATOR:

INTERACTIVE = True  

MASK = ''   # Make your own mask when you first CLEAN: 'POLCAL.mask'

#############





###################################################







mystep = 0

if(mystep in thesteps):

  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

  print 'Step ', mystep, step_title[mystep]



  os.system('rm -f %s.listobs'%VIS)

  listobs(vis=VIS, listfile='%s.listobs'%VIS)



  if os.path.exists(VIS+'.flagversions/flags.Original'):

    print 'Original flag versions already exists! Will not overwrite'

  else:

    flagmanager(vis=VIS,mode='save',versionname='Original')



  clearcal(vis=VIS)





########################################################

# Additional flagging:

mystep = 1

if(mystep in thesteps):

  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

  print 'Step ', mystep, step_title[mystep]





# ADD HERE WHATEVER YOU THINK IS BAD CALIBRATED DATA

########################################################

















# Execute always if there is a step number > 1:

if len([i for i in thesteps if i>1])>0:





###########################

# Figure out typical scan duration of the polcalib scans:



  ms.open(VIS)

  summary = ms.summary()

  ms.close()



  tb.open(VIS)

  scnum = tb.getcol('SCAN_NUMBER')

  stid = tb.getcol('STATE_ID')

  tb.close()



  tb.open(VIS+'/STATE')

  obsm = tb.getcol('OBS_MODE')

  tb.close()



  polst = [i for i in range(len(obsm)) if 'CALIBRATE_POLARIZATION' in obsm[i]]

  polscans = ['scan_%i'%i for i in np.unique(scnum[np.in1d(stid,polst)])]

  scandur = int(np.median([summary[ss]['0']['EndTime'] - summary[ss]['0']['BeginTime'] for ss in polscans])*86400.)

  print 'Will pre-average pol. data by %i seconds for calibration'%scandur



###########################











# Polarization calibration (X/Y Ratio):

mystep = 2

if(mystep in thesteps):

  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

  print 'Step ', mystep, step_title[mystep]





# Re-calibrate polarization calibrator:

  os.system('rm -rf %s.Gpol1'%VIS)



  gaincal(vis = VIS,

        caltable = '%s.Gpol1'%VIS,

        field = POLCAL,

        solint = 'inf',

##########

# CHOOSE:

#        gaintype = 'T',

        gaintype = 'G', 

##########

        refant = REFANT)



  os.system('rm -rf %s.Gpol1.png'%VIS)

  plotcal(VIS+'.Gpol1','scan','amp',field='',poln='/',subplot=111, figfile=VIS+'.Gpol1.png')







##########

# Save QU output in external file:

  orig_stdout = sys.stdout

  f = open('%s.QUfromGain.txt'%VIS, 'w')

  sys.stdout = f



# Rough estimate of QU:

  qu = aph.qufromgain('%s.Gpol1'%VIS)



# Save in outfile:

  ofile = open('qu.dat','w')

  pk.dump(qu,ofile)

  ofile.close()



  sys.stdout = orig_stdout

  f.close()



  f = open('%s.QUfromGain.txt'%VIS)

  print f.read()

  f.close()



##########











# JUST IN CASE THE STEP ABOVE IS NOT REPEATED:

# Read QU back:

if os.path.exists('qu.dat'):

  ifile = open('qu.dat')

  qu = pk.load(ifile)

  ifile.close()











# HOW CAN WE SEE THE CROSS-DELAY FROM THE DATA?

# USE PLOTMS!!





# Polarization calibration (Cross-delay):

mystep = 3

if(mystep in thesteps):

  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

  print 'Step ', mystep, step_title[mystep]







##################################################

# We search for the scan where the polarization signal is minimum in XX and YY

# (i.e., maximum in XY and YX):



  tb.open('%s.Gpol1'%VIS)

  scans = tb.getcol('SCAN_NUMBER')

  gains = np.squeeze(tb.getcol('CPARAM'))

  tb.close()

  scanlist = np.array(list(set(scans)))

  ratios = np.zeros(len(scanlist))

  for si, s in enumerate(scanlist):

    filt = scans == s

    ratio = np.sqrt(np.average(np.power(np.abs(gains[0,filt])/np.abs(gains[1,filt])-1.0,2.)))

    ratios[si] = ratio





  bestscidx = np.argmin(ratios)

  bestscan = scanlist[bestscidx]

  print 'Scan with highest expected X-Y signal: ', bestscan

#####################################################





# Save in outfile:

  ofile = open('xydelay.dat','w')

  pk.dump([scanlist,ratios,bestscidx,bestscan],ofile)

  ofile.close()





# Cross-hand delay:

  os.system('rm -rf %s.Kcrs'%VIS)

  gaincal(vis=VIS,

        caltable='%s.Kcrs'%VIS,

        selectdata=T,

        field = POLCAL,

        scan=bestscan,

        gaintype='KCROSS',

        solint='inf', # WHY!!!!

        refant=REFANT, 

        smodel = [1,0,1,0],  # WHY!!!!

        gaintable=['%s.Gpol1'%VIS],

        interp=['linear'])







# JUST IN CASE THE STEP ABOVE IS NOT REPEATED:

# Recover info:



if os.path.exists('xydelay.dat'):

  ifile = open('xydelay.dat')

  scanlist,ratios,bestscidx,bestscan = pk.load(ifile)

  ifile.close()















# Polarization calibration (Cross-phase):

mystep = 4

if(mystep in thesteps):

  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

  print 'Step ', mystep, step_title[mystep]







# XY phase offset:

  os.system('rm -rf %s.XY0amb'%VIS)

  gaincal(vis=VIS,

          caltable='%s.XY0amb'%VIS, 

          field= POLCAL,

          gaintype='XYf+QU', # WHY!!!!

          solint='inf',

          combine='scan,obs',

          preavg=scandur, 

          refant=REFANT,

          smodel=[1,0,1,0], # WHY!!!!

          gaintable=['%s.Gpol1'%VIS,'%s.Kcrs'%VIS],

          interp=['linear','nearest'])



  plotcal(caltable='%s.XY0amb'%VIS, xaxis='chan', yaxis='phase', 

          poln='/', figfile = '%s.XY0amb.png'%VIS, 

          plotrange=[0,0,-180,180])



  print 'END OF GAINCAL'

























# Polarization calibration (Cross-phase). Break ambiguity:

mystep = 5

if(mystep in thesteps):

  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

  print 'Step ', mystep, step_title[mystep]





# Solve QU ambiguity:

  orig_stdout = sys.stdout

  f = open('%s.XY-Ambiguity.txt'%VIS, 'w')

  sys.stdout = f



  os.system('rm -rf %s.XY0'%VIS)

  S = aph.xyamb(xytab='%s.XY0amb'%VIS, qu=qu.values()[0], xyout='%s.XY0'%VIS)



  sys.stdout = orig_stdout

  f.close()



  f = open('%s.XY-Ambiguity.txt'%VIS)

  print f.read()

  f.close()







#  ERIS2017.plot_B('%s.XY0amb'%VIS,'%s.XY0'%VIS,toplot=['cross-phase'])

  plot_B('%s.XY0amb'%VIS,'%s.XY0'%VIS,toplot=['cross-phase'])







##############

# HOW MUCH RESIDUAL POLARIZATION REMAINS AFTER THIS CALIBRATION??



## Re-calibrate polarization calibrator (with right pol. model): 

  os.system('rm -rf %s.Gpol2'%VIS) 



  gaincal(vis=VIS,

        caltable='%s.Gpol2'%VIS, 

        field=POLCAL,

        solint='inf',

        refant=REFANT,

        gaintype = 'G', 

        smodel=S,  # WHY!!!!

        parang=T)  # WHY!!!!





  os.system('rm -rf %s.Gpol2.png'%VIS)

  plotcal(VIS+'.Gpol2','scan','amp',field='',poln='/',subplot=111, figfile=VIS+'.Gpol2.png')





##########

## Check for any residual polarization signal:



  orig_stdout = sys.stdout

  f = open('%s.QUfromGain.txt'%VIS, 'a')

  f.write('\n\n USING POLCAL MODEL:\n\n')

  sys.stdout = f



  qu2 = aph.qufromgain('%s.Gpol2'%VIS)



  sys.stdout = orig_stdout

  f.close()



  f = open('%s.QUfromGain.txt'%VIS)

  print f.read()

  f.close()

##########









# Plot RMS of gain ratios around 1.0:



  tb.open('%s.Gpol2'%VIS)

  scans2 = tb.getcol('SCAN_NUMBER')

  gains = np.squeeze(tb.getcol('CPARAM'))

  tb.close()



  scanlist2 = np.array(list(set(scans2)))

  ratios2 = np.zeros(len(scanlist2))

  for si, s in enumerate(scanlist2):

    filt = scans2 == s

    ratio = np.sqrt(np.average(np.power(np.abs(gains[0,filt])/np.abs(gains[1,filt])-1.0,2.)))

    ratios2[si] = ratio





  os.system('rm -rf %s.GainRatiosPol.png'%VIS)

  pl.ioff()

  fig = pl.gcf()

  pl.clf() #pl.figure()

  sub = fig.add_subplot(111)

  sub.plot(scanlist, ratios, 'or', label = 'No Polcal')

  sub.plot(scanlist2, ratios2, 'ob', label = 'Polcal')

  sub.plot([scanlist[bestscidx]], [ratios[bestscidx]], 'xk', label = 'Best Pol. Scan')

  sub.set_xlabel('Scan Number')

  sub.set_ylabel('Gain Ratio RMS')

  pl.legend(numpoints=1)

  pl.savefig('%s.GainRatiosPol.png'%VIS)

  pl.clf()



  pl.ion()







  # Save outfile:

  ofile = open('S.dat','w')

  pk.dump(S,ofile)

  ofile.close()











# Recover info:



if os.path.exists('S.dat'):

  ifile = open('S.dat')

  S = pk.load(ifile)

  ifile.close()



























# Polarization calibration (Dterms):

mystep = 6

if(mystep in thesteps):

  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

  print 'Step ', mystep, step_title[mystep]









# Calibrate D-terms:



  os.system('rm -rf %s.Df0*'%VIS) 





  polcal(vis=VIS,

       caltable='%s.Df0'%VIS, 

       field= POLCAL, 

       solint='inf',

       combine='obs,scan',

       preavg=scandur,

       poltype='Dflls',

       refant='', #solve absolute D-term. WHY!!!!

       smodel=S,

       gaintable=['%s.Gpol2'%VIS, '%s.Kcrs'%VIS, '%s.XY0'%VIS],

       gainfield=['', '', ''],

       interp=['linear','nearest','linear'])



# Allow applying solutions to the parallel hands too:

  aph.Dgen(dtab='%s.Df0'%VIS, dout='%s.Df0gen'%VIS)





#  ERIS2017.plot_B('%s.Df0'%VIS,'%s.Df0gen'%VIS,toplot=['real','imag'])

  plot_B('%s.Df0'%VIS,'%s.Df0gen'%VIS,toplot=['real','imag'])





  plotcal(VIS+'.Df0gen','freq','real', spw='0',subplot=111,figfile=VIS+'.Df0gen.real.png')

  plotcal(VIS+'.Df0gen','freq','imag', spw='0',subplot=111,figfile=VIS+'.Df0gen.imag.png')



















# Polarization calibration (X/Y Amplitude Ratio):

mystep = 7

if(mystep in thesteps):

    casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

    print 'Step ', mystep, step_title[mystep]





# solve for global normalized gain amp (to get X/Y ratios) on pol calibrator

    os.system('rm -rf %s.Gxyamp*'%VIS)



# amp-only and normalized, so only X/Y amp ratios matter

    gaincal(vis=VIS,

        caltable='%s.Gxyamp'%VIS, 

        field=POLCAL,

        solint='inf',

        combine='scan,obs',

        refant=REFANT,

#########

# CHOOSE:

        gaintype='G',

#        gaintype='T',

        smodel=S,  # WHY!!!!

        calmode='a',

        gaintable=['%s.Kcrs'%VIS, '%s.XY0'%VIS, '%s.Df0gen'%VIS],

        gainfield=['', '', ''],

        interp=['nearest','linear','linear'],

        solnorm=T, # WHY!!!!

        parang=T) # WHY!!!!



    plotcal(VIS+'.Gxyamp', 'antenna','amp', spw='',

        iteration='spw',subplot=111,

        figfile='%s.Gxyamp.png'%VIS)

    plotcal(VIS+'.Gxyamp', 'antenna','amp', spw='',poln='/',

        iteration='spw',subplot=111,

        figfile='%s.GxyampRatio.png'%VIS)























# Save flags before applycal

mystep = 8

if(mystep in thesteps):

  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

  print 'Step ', mystep, step_title[mystep]



    

  Vname = 'BeforeApplycal'



  if os.path.exists('%s.flagversions/flags.%s'%(VIS,Vname)):

    print 'WILL DELETE PREVIOUS FLAG VERSION:\n%s'%Vname

    flagmanager(vis = VIS,

      mode = 'delete',

      versionname = Vname)

  

  flagmanager(vis = VIS,

    mode = 'save',

    versionname = Vname)



  







##############################

# FEEL FRE TO PLAY WITH THE XY0 AND Df0gen TABLES AS YOU LIKE!!!

#

IMNAME = VIS+'.CLEAN.POLCAL'

#os.system('rm -rf %s.Df0gen_PLAY  %s.XY0_PLAY'%(VIS,VIS))

#os.system('cp -r  %s.Df0gen  %s.Df0gen_PLAY'%(VIS,VIS))

#os.system('cp -r  %s.XY0 %s.XY0_PLAY'%(VIS,VIS))

#finetune_G('%s.Df0gen_PLAY'%VIS, XimagAdd = 0.01, YimagAdd = - 0.01)

#finetune_G('%s.XY0_PLAY'%VIS, XphaseAdd = 5.0)

#ERIS2017.finetune_G('%s.Df0gen_PLAY'%VIS, XimagAdd = 0.01, YimagAdd = - 0.01)

#ERIS2017.finetune_G('%s.XY0_PLAY'%VIS, XphaseAdd = 5.0)

#

##############################













# Application of the polarization calibration

mystep = 9

if(mystep in thesteps):

  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

  print 'Step ', mystep, step_title[mystep]



# Loop over sources and spws:

  for i in  [PHASECAL, POLCAL, TARGET]: 

    print 'Doing field %s'%i

    applycal(vis = VIS,

      field = i,

###########

# CHOOSE:

      gaintable = ['%s.Gxyamp'%VIS, 

                   '%s.Kcrs'%VIS, '%s.XY0'%VIS, '%s.Df0gen'%VIS],

#      gaintable = ['%s.Gxyamp'%VIS, 

#                   '%s.Kcrs'%VIS, '%s.XY0_PLAY'%VIS, '%s.Df0gen_PLAY'%VIS],

###########

      gainfield = [POLCAL, POLCAL, POLCAL, POLCAL],

      interp = ['nearest','nearest','nearest','nearest'],

      calwt = F,

      parang = T,  # WHY!!!!

      flagbackup = F)

  









# Save flags after applycal

mystep = 10

if(mystep in thesteps):

  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

  print 'Step ', mystep, step_title[mystep]





  Vname = 'AfterApplycal'



  if os.path.exists('%s.flagversions/flags.%s'%(VIS,Vname)):

    print 'WILL DELETE PREVIOUS FLAG VERSION:\n%s'%Vname

    flagmanager(vis = VIS,

      mode = 'delete',

      versionname = Vname)

  

  flagmanager(vis = VIS,

    mode = 'save',

    versionname = Vname)

  

  

















# CLEAN the pol. calibrator:



mystep = 11

if(mystep in thesteps):

  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

  print 'Step ', mystep, step_title[mystep]





  os.system('rm -rf %s.*'%IMNAME)

  clean(vis=VIS,

        field=POLCAL,

        imagename = IMNAME,

        cell = '0.025arcsec',

        imsize = 1024,

        psfmode = 'clarkstokes',

        stokes = 'IQUV',

        mode = 'mfs',

        niter = 1000,

        cyclefactor = 0.0,

        mask = MASK,

        interactive = INTERACTIVE)







mystep = 12

if(mystep in thesteps):

  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')

  print 'Step ', mystep, step_title[mystep]







#  ERIS2017.plotPolImage(IMNAME+'.image', IMNAME+'.residual', 

#                    plotrange=[8,-2,-7,3], sigmaCut=4.0, 

#                    NpixPol=5, polLength=0.1) 

  

  plotPolImage(IMNAME+'.image', IMNAME+'.residual', 

                    plotrange=[8,-2,-7,3], sigmaCut=4.0, 

                    NpixPol=5, polLength=0.1) 









